/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: StageCoordinator.java,v 1.3 2005/08/19 03:18:11 ppoi Exp $
 */
package tsukuba_bunko.peko.scenario.stage;

import	tsukuba_bunko.peko.ActionControler;
import tsukuba_bunko.peko.Logger;

import	tsukuba_bunko.peko.canvas.stage.Actor;
import	tsukuba_bunko.peko.canvas.stage.Stage;

import tsukuba_bunko.peko.scenario.Coordinator;
import	tsukuba_bunko.peko.scenario.SceneContext;


/**
 * StageCanvas ւ̑莝R[fBl[^W[łB
 * @author	$Author: ppoi $
 * @version	$Revision: 1.3 $ $Date: 2005/08/19 03:18:11 $
 */
public class StageCoordinator	extends Coordinator	{

	/**
	 * ʒuF
	 */
	public static final int	POSITION_CENTER = 0;

	/**
	 * ʒuF
	 */
	public static final int	POSITION_LEFT = 1;

	/**
	 * ʒuFE
	 */
	public static final int	POSITION_RIGHT = 2;


	/**
	 * lXgx
	 */
	protected int	_level = 0;


	/**
	 * <code>StageCoordinator</code> ̃CX^X𐶐܂B
	 */
	public StageCoordinator()
	{
		super();
	}


	/**
	 * <code>name</code> Ŏʂ҂̏擾܂B
	 * @param	name	Җ
	 * @return	Actor IuWFNgBY҂Ȃꍇ <code>null</code>
	 */
	public Actor getActor( String name )
	{
		Actor	actor = getStage().getActor( name );
		if( actor == null )	{
			return null;
		}
		else	{
			Actor	copy = new Actor( actor.getName() );
			actor.copyTo( copy );
			return copy;
		}
	}


	/**
	 * ҂𕑑ɓoꂳ܂B
	 * @param	actor	oꂷ
	 */
	public void enter( Actor actor )
	{
		if( isActiveThread() )	{
			getStage().enter( actor );
		}
	}

	/**
	 * ҂𓮂܂B
	 * @param	actor	
	 */
	public void action( Actor actor )
	{
		if( isActiveThread() )	{
			Stage	stage = getStage();
			Actor	current = stage.getActor( actor.getName() );
			if( current != null )	{
				if( current.getLooksImage() != actor.getLooksImage() )	{
					stage.exit( actor.getName() );
					stage.enter( actor );
				}
				else	{
					stage.enter( actor );
				}
			}
		}
	}

	/**
	 * ҂ޏꂳ܂B
	 * @param	name	ޏꂳҖ
	 * @return	ޏꂵ
	 */
	public Actor exit( String name )
	{
		if( isActiveThread() )	{
			return getStage().exit( name );
		}
		else	{
			return null;
		}
	}

	/**
	 * wiw肵Fɐݒ肵܂B
	 * @param	color	F
	 */
	public void setBackgroundColor( String color )
	{
		if( isActiveThread() )	{
			getStage().setBackgroundColor( color );
		}
	}

	/**
	 * wiw肳ꂽ摜ɐݒ肵܂B
	 * @param	image	摜
	 */
	public void setBackgroundImage( String image )
	{
		if( isActiveThread() )	{
			getStage().setBackgroundImage( image );
		}
	}

	/**
	 * XCh\܂B
	 * @param	slide	XCh
	 */
	public void showSlide( String slide )
	{
		if( isActiveThread() )	{
			getStage().showSlide( slide );
		}
	}

	/**
	 * XChB܂B
	 */
	public void hideSlide()
	{
		if( isActiveThread() )	{
			getStage().hideSlide();
		}
	}

	/**
	 * BGM Đ܂B
	 * @param	id	BGM ID
	 * @param	clipName	Nbv
	 * @param	loop	[vꍇ <code>true</code>AȊȌꍇ <code>false</code>
	 */
	public void playBGM( String id, String clipName, boolean loop )
	{
		if( isActiveThread() )	{
			getStage().playBGM( id, clipName, loop );
		}
	}

	/**
	 * BGM ~܂B
	 * @param	id	BGM ID
	 */
	public void stopBGM( String id, int mode )
	{
		if( isActiveThread() )	{
			getStage().stopBGM( id, mode );
		}
	}

	/**
	 * SE Đ܂B
	 * @param	id	SE ID
	 * @param	clipName	Nbv
	 * @param	loop	[vꍇ <code>true</code>AȊȌꍇ <code>false</code>
	 */
	public void playSE( String id, String clipName, boolean loop )
	{
		if( isActiveThread() )	{
			getStage().playSE( id, clipName, loop );
		}
	}

	/**
	 * SE ~܂B
	 * @param	id	SE ID
	 */
	public void stopSE( String id, int mode )
	{
		if( isActiveThread() )	{
			getStage().stopSE( id, mode );
		}
	}

	/**
	 * Xe[Ww肳ꂽGtFNggpčXV܂B
	 * @param	effect	GtFNg
	 */
	public void updateStage( String effect )
	{
		Logger.debug( "[scenario.stage] update stage. effect=" + effect );
		if( _level > 1 )	{
			return;
		}

		Stage	stage = getStage();
		stage.updateCanvas( effect );
	}

	/**
	 * GtFNggpɃXe[WXV܂B
	 */
	public void updateStage()
	{
		if( isActiveThread() )	{
			updateStage( null );
		}
	}

	/**
	 * eLXgLoXB܂B
	 */
	public void hideTextCanvas()
	{
		if( isActiveThread() )	{
			getCanvasManager().hideTextCanvas();
		}
	}

	/**
	 * Xe[W擾܂B
	 * @return	Xe[W
	 */
	protected Stage getStage()
	{
		return getCanvasManager().getStage();
	}

	/**
	 * XCh\ǂ𔻒肵܂B
	 * @return	XCh\̏ꍇ <code>true</code>C\̏ꍇ <code>false</code>
	 */
	public boolean isSlideVisible()
	{
		return (getStage().getSlideImage() != null);
	}


//
//	Coordinator ̎
//
	public void prepare( SceneContext context, Thread activeThread )
	{
		super.prepare( context, activeThread );
		_level = 0;
	}

	/**
	 * StageCanvas ɑ΂鑀Jn܂B
	 */
	public void begin()
	{
		if( isActiveThread() )	{
			_level++;
			getCanvasManager().hideTextCanvas();
			getActionControler().setActive( false );
			getActionControler().setSaveEnabled( true );
		}
	}

	/**
	 * StageCanvas ɑ΂鑀IALoX̏Ԃm肵܂B
	 */
	public void commit()
	{
		if( isActiveThread() )	{
			_level--;
			if( _level == 0 )	{
				ActionControler	controler = getActionControler();
				controler.setActive( true );
				controler.setSaveEnabled( false );
				Stage	stage = getStage();
				stage.commit();
			}
		}
	}
}